/*
 * Project: Bluetooth Net Monitor
 * Author: Zak Kemble, contact@zakkemble.co.uk
 * Copyright: (C) 2013 by Zak Kemble
 * License: GNU GPL v3 (see License.txt)
 * Web: http://blog.zakkemble.co.uk/bluetooth-net-monitor-v2/
 */

#include <regex.h>
#include "typedefs.h"
#include "adsl.h"
#include "updater.h"

static void processData(s_memoryChunk*);

static s_updateData* uptData;

void adsl_init()
{
	updater_construct(
		&uptData,
		"adslpage",
		"adsluser",
		"adslpass",
		"adslregex",
		"adslcachetime",
		&processData,
		sizeof(s_netUpDown)
		);
}

void adsl_update()
{
	updater_run(uptData);
}

s_netUpDown adsl_get()
{
	return *(s_netUpDown*)uptData->data;
}

void adsl_end()
{
	updater_destroy(&uptData);
}

static void processData(s_memoryChunk* chunk)
{
	regmatch_t matches[3];

	int regexRes = regexec(&uptData->reg, chunk->memory, 3, matches, 0);
	if(regexRes == 0)
	{
		((s_netUpDown*)uptData->data)->down = updater_getMatchNum(&matches[1], chunk);
		((s_netUpDown*)uptData->data)->up = updater_getMatchNum(&matches[2], chunk);
	}
}